


------------------------------------
---    data type define
------------------------------------

-- enum
PlayOrder = {
    Simultaneously  = 0,
    Randomly        = 1,
    Sequentially    = 2,
    PickOneRandomly = 3
}


AnimationAction = {
    Disappear       = 0,
    Appear          = 1
}

AnimationInType = {
    null = 0,                           --    无
    OpacityInOut = 1,                   --    渐隐渐显
    FlyInOutUpside = 2,                 --    飞入飞出（从上）
    FlyInOutDownside = 3,               --    飞入飞出（从下）
    FlyInOutLeftside = 4,               --    飞入飞出（从左）
    FlyInOutRightside = 5,              --    飞入飞出（从右）
    RotateInOutClockwise = 6,           --    顺时针旋转
    RotateInOutCounterClockwise = 7,    --    逆时针旋转
    ZoomOut = 8,                        --    缩小
    ZoomIn = 9                          --    放大
}

AnimationOutType = {
    unknown = -1,
    anchor = 0,
    position = 1,
    scale = 2,
    rotation = 3,
    opacity = 4,
    transform_num = 5
}

ValidPeriod = {
    Indefinite      = 0,
    DefineTime      = 1,
    DefineCount     = 2
}

TrigCondition = {
    And             = 0,
    Or              = 1
}

TrigType = {
    None            = 0,
    Face            = 1,
    Hand            = 2,
}

TrigSubTypeFace = {
    -- trigger by face
        Others            = 0,
        CloseLeftEye      = 1,
        CloseRightEye     = 2,
        RaiseLeftEyebrow  = 3,
        RaiseRightEyebrow = 4,
        MouthOpen         = 5,
        Pout              = 6,
        Smile             = 7,
        Sad               = 8,
        Laugh             = 9,
        WinkLeftRye       = 10,
        WinkRightEye      = 11,
        PoutNew           = 12,
        SadNew            = 13,
        Angry             = 14,
        TongueOut         = 15,
        Cry               = 16,
        MouthOpenNew      = 17,
        SmileNew          = 18
}

TrigSubTypeHand = {
    -- trigger by hand
    Appear          = 0,
    Disappear       = 1
}

--EventCallback
CallBack = {
    sublayer = nil,
    params = {}
}

function CallBack:new (o,subLayer,params)
    o = o or {}
    setmetatable(o, self)
    self.__index = self
    o.sublayer = subLayer or nil
    o.params = params or {}
    return o
end

function CallBack:print()
    print("subLayer = ",self.sublayer)
    print("params",self.params)
end

--TransAnimation
TransAnimationElement = {
    transAnimationType = 0,
    transAnimationString = ""
}

function TransAnimationElement:new (o)
    o = o or {}
    setmetatable(o, self)
    self.__index = self
    return o
end

-- NodeEffectElement
NodeEffectElement = {

    type = "",
    name = "",
    --- animation
    animationAction = AnimationAction.Appear,
    transAnimationInArray = {},
    transAnimationOutArray = {},
    --- play
    startFrame = 0,
    endFrame = 1,
    validPeriod = ValidPeriod.Indefinite,
    validPeriodParam = 0,

    --- after play
    stopAtLastFrame = true,
    keepForever = true ,
    keepDuration = 1  -- seconds
}

function NodeEffectElement:new (o)
    o = o or {}
    setmetatable(o, self)
    self.__index = self
    return o
end


-- NodeEffectModel
NodeEffectModel = {
    eventType = 1,
    playOrder = PlayOrder.Simultaneously,
    effectElements = {}  -- array of NodeEffectElement
}
function NodeEffectModel:new(o)
    o = o or {}
    setmetatable(o, self)
    self.__index = self
    return o
end


-- NodeTriggerElement
NodeTriggerElement = {
    name = "",
    trigType = TrigType.Face,
    trigSubType = TrigSubTypeFace.CloseLeftEye
}
function NodeTriggerElement:new(o)
    o = o or {}
    setmetatable(o, self)
    self.__index = self
    return o
end

-- NodeTriggerGroup
NodeTriggerGroup = {
    eventType = 2,
    trigCondition = TrigCondition.Or,
    portId = "",
    triggerElements = {}  -- array of NodeTriggerElement
}
function NodeTriggerGroup:new(o)
    o = o or {}
    setmetatable(o, self)
    self.__index = self
    return o
end

-- NodeTriggerModel
NodeTriggerModel = {
    triggerGroups = {} -- array of NodeTriggerGroup
}
function NodeTriggerModel:new(o)
    o = o or {}
    setmetatable(o, self)
    self.__index = self
    return o
end


-- LuaNode
LuaNode = {
    id = "",
    nodename = "",
    children = {},  -- array of string
    effectModel = {}, -- object of NodeEffectElement
    triggerModel = {} -- object of NodeTriggerModel
}
function LuaNode:new(o)
    o = o or {}
    setmetatable(o, self)
    self.__index = self
    return o
end

-- Connection node
ConnectionPair = {
    startNode = "",      -- node id
    startNodePort = "",  -- start group port id
    endNode = ""        -- end node id
}
function ConnectionPair:new(o)
    o = o or {}
    setmetatable(o, self)
    self.__index = self
    return o
end

-- luanetwork
LuaNetwork = {
    nodeData = {},  -- array of LuaNode
    connections = {} -- array of ConnectionPair
}
function LuaNetwork:new(o)
    o = o or {}
    setmetatable(o, self)
    self.__index = self
    return o
end



------------------------------------
---    network logic process
------------------------------------
-- Event enum
Event = {
    SetEnable           = 1,
    SetDisable          = 2,
    Stop                = 3,
    AnimationForward    = 4,
    AnimationBackWard   = 5,
    AnimationReset      = 6
}

local fps = 30
local effectStateTable = {}
local currentPortPool = {}    -- currentPortPool["{PortId}"] = {trigSubtype_1,trigSubtype_2}

local EventQueue = {}
function EventQueue:new(o)
    o = o or {}
    setmetatable(o, self)
    self.__index = self
    return o
end
function EventQueue:append(index, event ,layer)
    if(self[index] == nil) then
        self[index] = {}
    end
    if((self[index][event] == nil)) then
        self[index][event] = {layer}
    else
        table.insert(self[index][event], layer)
    end
end

local eventQueue = EventQueue:new(nil)

local isInPreview = true

function printTab(tab)
    for i,v in pairs(tab) do
        print(i,v)
    end
end

function removeElementByKey(tbl,key)
    --新建一个临时的table
    local tmp ={}

    --把每个key做一个下标，保存到临时的table中，转换成{1=a,2=c,3=b}
    --组成一个有顺序的table，才能在while循环准备时使用#table
    for i in pairs(tbl) do
        table.insert(tmp,i)
    end

    local newTbl = {}
    --使用while循环剔除不需要的元素
    local i = 1
    while i <= #tmp do
        local val = tmp [i]
        if val == key then
            --如果是需要剔除则remove
            table.remove(tmp,i)
         else
            --如果不是剔除，放入新的tabl中
            newTbl[val] = tbl[val]
            i = i + 1
         end
     end
    return newTbl
end

function randomTable(_table, _num)
    local _result = {}
    local _index = 1
    local _num = _num or #_table
    while #_table ~= 0 do
        local ran = math.random(0, #_table)
        if _table[ran] ~= nil then
            _result[_index] = _table[ran]
            table.remove(_table,ran)
            _index = _index + 1
            if _index > _num then
                break
            end
        end
    end
    return _result
end

function AnimationInTypeOut(animInType)
    if(animInType == AnimationInType.OpacityInOut) then
        return AnimationOutType.opacity
    elseif (animInType >= AnimationInType.FlyInOutUpside and animInType <= AnimationInType.FlyInOutRightside) then
        return AnimationOutType.position
    elseif (animInType == AnimationInType.RotateInOutClockwise or animInType == AnimationInType.RotateInOutCounterClockwise) then
        return AnimationOutType.rotation
    elseif (animInType == AnimationInType.ZoomOut or animInType == AnimationInType.ZoomIn) then
        return AnimationOutType.scale
    else
        return AnimationOutType.unknown
    end
end

queueManager = {
    takePhotoBegin = function(this,eventCode,eventMessage)
        print('takePhoto SLREEvent_TAKE_PHOTO_BEGIN')
        if not isInPreview then
            return
        end

        initEffects = this:getLayerByName("2DSticker_init")
        initEffects:setEnable(false)
        networkHandler.initQueue(this,eventMessage)

        isInPreview = false
    end
    ,
    takePhotoDismiss = function(this,eventCode,eventMessage)
        print('takePhoto end')
        if isInPreview then
            return
        end

        initEffects = this:getLayerByName("2DSticker_init")
        initEffects:setEnable(true)
        networkHandler.releaseQueue()

        isInPreview = true
    end
    ,
    videoRecordBegin = function(this, eventCode, eventMessage)
        print('record begin')
        if not isInPreview then
            return
        end

        initEffects = this:getLayerByName("2DSticker_init")
        initEffects:setEnable(false)
        networkHandler.initQueue(this,eventMessage)

        isInPreview = false
    end
    ,
    videoRecordDismiss = function(this, eventCode, eventMessage)
        print('record end')
        if isInPreview then
            return
        end

        initEffects = this:getLayerByName("2DSticker_init")
        initEffects:setEnable(true)
        networkHandler.releaseQueue()

        isInPreview = true
    end
}

networkHandler = {

    initQueue = function(this,eventMessage)
        currentPortPool = {}
        eventQueue = EventQueue:new(nil)

        -- assert(#luaNetwork.nodeData == 0, "Entered An Empty Event")

        ---- find root nodes
        networkHandler.findRootNode(this,eventMessage)
    end
    ,
    releaseQueue = function()
        currentPortPool = nil
        eventQueue = nil
    end
    ,
    findRootNode = function(this,eventMessage)
        local chilrenAssem = {}
        for i,v in pairs(luaNetwork.connections) do
            --print("endnode----------",v.endNode)
            table.insert(chilrenAssem,v.endNode)
        end
        print("chilrenAssem----------")
        printTab(chilrenAssem)
        for i,v in pairs(luaNetwork.nodeData) do
            if (#chilrenAssem == 0) then          ---- no connections
                local node = networkHandler.getNodeByNodeId(v.id)
                effectHandler.setEffectModelParams(this,node,eventMessage)
            elseif (v.nodename == "root") then    ---- circle event
                local node = networkHandler.getNodeByNodeId(v.id)
                effectHandler.setEffectModelParams(this,node,eventMessage)
                for index, value in pairs(v.triggerModel.triggerGroups) do
                    currentPortPool[value.portId] = {}
                end
            else
                print("nodeID------- ",v.id)
                local isAChild = false
                for index, value in pairs(chilrenAssem) do
                    if (v.id == value) then
                        isAChild = true
                        break
                    end
                end
                if(isAChild == false) then
                    print("find root node -----",v.id)
                    local node = networkHandler.getNodeByNodeId(v.id)
                    effectHandler.setEffectModelParams(this,node,eventMessage)
                    for index, value in pairs(v.triggerModel.triggerGroups) do
                        currentPortPool[value.portId] = {}
                    end
                end
            end
        end
    end
    ,
    getNodeByNodeId = function (id)
        -- empty event
        -- assert(luaNetwork.nodeData == nil, "Entered An Empty Event " .. #(luaNetwork.nodeData))
        for i,v in pairs(luaNetwork.nodeData)
        do
            --print("???",v.id,"      ",id)
            if(v.id == id) then
                return v
            end
        end
        assert(true,"Unknown NodeId")
    end
    ,
    getNodeByPortId = function(id)
        for key,value in pairs(luaNetwork.nodeData)
        do
            for k,v in pairs(value.triggerModel.triggerGroups)
            do
                if(v.portId == id) then
                    return value
                end
            end
        end
        assert(true,"Unknown PortId")
    end
    ,
    getChildNodebyPortId = function (id)
        print("connections------- ",#luaNetwork.connections)
        for key, value in pairs(luaNetwork.connections) do
            print("id-----------",id,"     ",value.startNodePort)
            if(id == value.startNodePort) then
                return networkHandler.getNodeByNodeId(value.endNode)
            end
        end
    end
    ,
    getPortsByNodeId = function (id)
        local ports = {}
        local node =  networkHandler.getNodeByNodeId(id)
        --print("node----------  ",node)
        -- leaf node
        local triggerGroups = node.triggerModel.triggerGroups
        print("triggerGroups------ ",#triggerGroups)
        if(#triggerGroups == 0) then
            return ports
        end
        for i,v in pairs(triggerGroups)
        do
            table.insert(ports,v.portId)
        end

        return ports
    end
    ,
    getChildPortsByPortId = function (id)
        local endNode = ""
        --print("1 ---------", #luaNetwork.connections)
        --print("2---------------",id)
        -- no connections
        if(#luaNetwork.connections == 0) then
            return {}
        end

        for i,v in pairs(luaNetwork.connections)
        do
            --print("3-------------",id,'     ',v.startNodePort)
            if(v.startNodePort == id)   then
                endNode = v.endNode
                break
            end
        end
        --print("4-------------",endNode)
        return networkHandler.getPortsByNodeId(endNode)
    end
    ,
    getTriggerGroupByPortId = function (id)
        -- cannot use connection search for leaf ports
        -- port already exsists
        for key,value in pairs(luaNetwork.nodeData)
        do
            for k,v in pairs(value.triggerModel.triggerGroups)
            do
                if(v.portId == id) then
                    return v
                end
            end
        end
        assert(true,"Unknown PortId")
    end

}
local flfag = true
effectHandler = {
    newframeHandle = function(this,eventMessage)            ----  set subLayer enable / disable / stop
        local ensuranceCoefficient = 5                     ---- process specific lost frame event
        local frameId = eventMessage.frameId

        if(flfag) then
            networkHandler.initQueue(this,eventMessage)
            flfag = false
        end

        --print("eventQueue-----------")
        --printTab(eventQueue)
        --print("eventQueue-----------")
        --print("currentPortPool----------")
        --printTab(currentPortPool)
        --print("currentPortPool----------")
        --print("frameId ---- ",frameId)

        effectHandler.processFrameEvent(frameId,ensuranceCoefficient)

        -- if(frameId == 200 ) then
        --     EventHandles.handleEffectEvent(this,202, eventMessage)
        -- end

    end
    ,
    processFrameEvent = function(frameId,ensuranceCoefficient)
        local coefficient = 0
        if (frameId > ensuranceCoefficient) then
            coefficient = ensuranceCoefficient
        else
            coefficient = frameId - 1
        end
        for i = coefficient,0,-1 do
            if (eventQueue[frameId - i] ~= nil) then

                if (eventQueue[frameId - i][Event.SetDisable]) then
                    for index,v in pairs(eventQueue[frameId - i][Event.SetDisable]) do
                        print("getAllAnimationPlaying-----",frameId - i,v:getAllAnimationPlaying(0))
                        if (effectStateTable[v:getName()] ~= nil and  effectStateTable[v:getName()]["disable"] == frameId - i) then
                            v:setEnable(false)

                            print("setDisable-------- ",#eventQueue[frameId - i][Event.SetDisable],"frame =   ",frameId - i)
                        end
                    end
                end
                if (eventQueue[frameId - i][Event.SetEnable]) then
                    print("setenable--------   ",#eventQueue[frameId - i][Event.SetEnable])
                    for index,v in pairs(eventQueue[frameId - i][Event.SetEnable]) do
                        v:setEnable(true)
                    end
                end
                if (eventQueue[frameId - i][Event.Stop]) then
                    for index,v in pairs(eventQueue[frameId - i][Event.Stop]) do
                        v:setClipAutoRepeat(false)
                    end
                end
                if (eventQueue[frameId - i][Event.AnimationForward]) then
                    for index,v in pairs(eventQueue[frameId - i][Event.AnimationForward]) do
                        local sublayer = v.sublayer
                        local params = v.params["AnimationType"]
                        print("animation in----",sublayer:getName())
                        print("lua setAnimationState-----",0,params,0)
                        sublayer:setAnimationState(0,params,0)
                    end
                end
                if (eventQueue[frameId - i][Event.AnimationBackWard]) then
                    for index,v in pairs(eventQueue[frameId - i][Event.AnimationBackWard]) do
                        print("setAnimationBackWard--------   ",frameId - i,i)
                        local sublayer = v.sublayer
                        local params = v.params["AnimationType"]
                        print("animation out----",sublayer:getName())
                        print("lua setAnimationState-----",0,params,1)
                        sublayer:setAnimationState(0,params,1)
                    end
                end
                if (eventQueue[frameId - i][Event.AnimationReset]) then
                    for index,v in pairs(eventQueue[frameId - i][Event.AnimationReset]) do
                        local sublayer = v.sublayer
                        local params = v.params["AnimationType"]
                        print("getAnimationPlaying-----",sublayer:getAnimationPlaying(0,params),params)
                        if (not sublayer:getAnimationPlaying(0,params)) then
                            sublayer:resetAnimation(0,params)

                            --local frameCount = sublayer:getAnimationFrameCount(0,params)
                            --sublayer:setAnimationFrameID(0,params,frameCount - 1)

                            sublayer:setAnimationState(0,params,2)
                            print("animation reset----",0,params,2)
                        end
                    end
                end
                eventQueue[frameId - i] = nil
            end
        end
    end
    ,
    setEffectModelParams = function(this,node,eventMessage)     -- set layer peoperties and add to queues
        if(node ~= nil) then
            local effectModel = node.effectModel
            local frameId = 1
            if(eventMessage) then
                frameId = eventMessage.frameId
            end

            if(#effectModel.effectElements > 0) then
                if(effectModel.playOrder == PlayOrder.Simultaneously) then
                    effectHandler.effectModelSimultaneouslyPlay(this,effectModel.effectElements,frameId)
                elseif(effectModel.playOrder == PlayOrder.Sequentially) then
                    effectHandler.effectModelSequentiallyPlay(this,effectModel.effectElements,frameId)
                elseif(effectModel.playOrder == PlayOrder.Randomly) then
                    effectHandler.effectModelRandomlyPlay(this,effectModel.effectElements,frameId)
                else
                    effectHandler.effectModelRandomlyPlayOnce(this,effectModel.effectElements,frameId)
                end
            end
        end
    end
    ,

    -- set params of facesticker/fg effect element
    setStickerEffect = function(effectElement,subLayer,frameId)
        local duration = 0
        duration = effectHandler.setAnimationEffect(effectElement,subLayer,frameId)
        if(effectElement.animationAction == AnimationAction.Appear) then

            subLayer:setClipEpochFrame(frameId)
            --subLayer:setClipPlayRange(effectElement.startFrame,effectElement.endFrame)

            --setvalidPeriod
            subLayer:setClipAutoRepeat(true)
            eventQueue:append(frameId,Event.SetEnable,subLayer)

            local fps_video = subLayer:getVideoFPS() --subLayer.getClipFrameCount()
            local fps_output = subLayer:getAssetFPS()--subLayer.getRangeClipFrameCount()
            local frameCounts = subLayer:getClipFrameCount()
            --print("fps_output-------  ",fps_output)
            --print("fps_video-------  ",fps_video)
            --print("frameCounts-------  ",frameCounts)
            --print("effectElement.validPeriodParam-------  ",effectElement.validPeriodParam)
            if(effectElement.validPeriod == ValidPeriod.Indefinite) then
                --do nothing
                duration = 0
            elseif(effectElement.validPeriod == ValidPeriod.DefineTime) then
                duration = fps_video * effectElement.validPeriodParam
                eventQueue:append(frameId + duration, Event.SetDisable, subLayer)
            else
                duration = effectElement.validPeriodParam * frameCounts  --(frameCounts + 1)* (fps_video / fps_output)
                if(effectElement.stopAtLastFrame) then
                    eventQueue:append(frameId + duration , Event.Stop, subLayer)

                    if(effectElement.keepForever) then
                        --only need to set unrepeative
                    else
                        --need to be set disabled
                        eventQueue:append(frameId + duration + effectElement.keepDuration * fps_video, Event.SetDisable, subLayer)
                    end
                else
                    eventQueue:append(frameId + duration, Event.SetDisable, subLayer)
                end
            end
            local effectState = {enable = frameId,disable = frameId + duration}
            effectStateTable[subLayer:getName()] = effectState
        else
            eventQueue:append(frameId + duration, Event.SetDisable, subLayer)
            local effectState = {enable = frameId,disable = frameId + duration}
            effectStateTable[subLayer:getName()] = effectState
        end

        return duration
    end
    ,
    -- set params of filter effect element
    setLutEffect = function(effectElement,subLayer,frameId)
        print("sublayer-------    ",subLayer)

        if (effectElement.animationAction == AnimationAction.Appear) then
            eventQueue:append(frameId,Event.SetEnable,subLayer)
            local effectState = {enable = frameId,disable = nil}
            effectStateTable[subLayer:getName()] = effectState
        else
            eventQueue:append(frameId,Event.SetDisable,subLayer)
            local effectState = {enable = nil,disable = frameId}
            effectStateTable[subLayer:getName()] = effectState
        end
    end
    ,
    setMakeupEffect = function (effectElement,subLayer,frameId)
        local duration = 0
        duration = effectHandler.setAnimationEffect(effectElement,subLayer,frameId)
        if (effectElement.animationAction == AnimationAction.Appear) then
            eventQueue:append(frameId,Event.SetEnable,subLayer)
            local effectState = {enable = frameId,disable = nil}
            effectStateTable[subLayer:getName()] = effectState
        else
            eventQueue:append(frameId + duration,Event.SetDisable,subLayer)
            local effectState = {enable = nil,disable = frameId + duration}
            effectStateTable[subLayer:getName()] = effectState
        end
    end
    ,
    setAnimationEffect = function (effectElement,subLayer,frameId)
        local duration = 0
        if(effectElement.animationAction == 1) then
            print("#effectElement.transAnimationInArray-----",#effectElement.transAnimationInArray)
            for index, value in ipairs(effectElement.transAnimationInArray) do
                if (value.transAnimationType ~= 0 and value.transAnimationString.len ~=0) then
                    subLayer:buildAnimation(value.transAnimationString,fps)
                    local animOutType = AnimationInTypeOut(value.transAnimationType)
                    subLayer:resetAnimation(0,animOutType)
                    print("lua init anim in")
                    print("lua setAnimationState-----",0,animOutType,2)
                    subLayer:setAnimationState(0,animOutType,2)
                    local layer = CallBack:new({},subLayer,{AnimationType = animOutType})
                    eventQueue:append(frameId,Event.AnimationForward,layer) --延时播放准备
                end
            end
        else
            print("#effectElement.transAnimationOutArray-----",#effectElement.transAnimationOutArray)
            for index, value in ipairs(effectElement.transAnimationOutArray) do
                if (value.transAnimationType ~= 0 and value.transAnimationString.len ~=0) then
                    subLayer:buildAnimation(value.transAnimationString,fps)
                    local animOutType = AnimationInTypeOut(value.transAnimationType)
                    local frameCount = subLayer:getAnimationFrameCount(0,animOutType)
                    duration = math.max(duration,frameCount)
                    subLayer:setAnimationFrameID(0,animOutType,frameCount - 1)
                    print("lua init anim out")
                    print("lua setAnimationState-----",0,animOutType,2)
                    print("lua anim framecoutn-----",frameCount)
                    subLayer:setAnimationState(0,animOutType,2)
                    local layer = CallBack:new({},subLayer,{AnimationType = animOutType})
                    eventQueue:append(frameId,Event.AnimationBackWard,layer) --延时播放准备
                    eventQueue:append(frameId + duration,Event.AnimationReset,layer)
                end
            end
        end
        return duration
    end
    ,
    effectModelSimultaneouslyPlay = function(this,effectElements,frameId)
        for i,v in pairs(effectElements) do
            print("stickername-----",v.name)
            print("layers--------",#this:getLayers())
            for j,t in pairs(this:getLayers()) do
                print("layers--------",t:getName())
                --print("sublayers--------",t:getLayers()[1]:getName())
            end
            local curSubLayer = this:getLayerByName(v.name)
            print("curSubLayer--------",curSubLayer)
            if(v.type == "FaceStickerElement" or v.type == "ForeStickerElement" or v.type == "CatStickerElement" or v.type == "DogStickerElement") then
                effectHandler.setStickerEffect(v,curSubLayer,frameId)
            elseif (v.type == "BeautyMakeupElement") then
                effectHandler.setMakeupEffect(v,curSubLayer,frameId)
            else
                effectHandler.setLutEffect(v,curSubLayer,frameId)
            end
        end
    end
    ,
    effectModelSequentiallyPlay = function(this,effectElements,frameId)
        local playFrames = 0
        local duration = 0
        for i, v in pairs(effectElements) do
            local curSubLayer = this:getLayerByName(v.name)
            if(v.type == "FaceStickerElement" or v.type == "ForeStickerElement" or v.type == "CatStickerElement" or v.type == "DogStickerElement") then
                duration = effectHandler.setStickerEffect(v,curSubLayer,frameId)
                if(duration) then
                    frameId = frameId + duration
                end
            elseif (v.type == "BeautyMakeupElement") then
                duration = effectHandler.setMakeupEffect(v,curSubLayer,frameId)
                if(duration) then
                    frameId = frameId + duration
                end
            else
                effectHandler.setLutEffect(v,curSubLayer,frameId)
            end
        end
    end
    ,
    effectModelRandomlyPlay = function(this,effectElements,frameId)
        -- create a random table
        local rdmTbl = {}
        local playFrames = 0
        for i = 1,#effectElements do
            table.insert(rdmTbl,i)
        end
        rdmTbl = randomTable(rdmTbl)
        local duration = 0
        for i,v in pairs(rdmTbl) do
            local effectElement = effectElements[v]
            local curSubLayer = this:getLayerByName(effectElement.name)
            if(effectElement.type == "FaceStickerElement" or
                effectElement.type == "ForeStickerElement" or
                effectElement.type == "CatStickerElement" or
                effectElement.type == "DogStickerElement" ) then
                duration = effectHandler.setStickerEffect(effectElement,curSubLayer,frameId)
                frameId = frameId + duration
            elseif (effectElement.type == "BeautyMakeupElement") then
                duration = effectHandler.setMakeupEffect(effectElement,curSubLayer,frameId)
                if(duration) then
                    frameId = frameId + duration
                end
            else
                effectHandler.setLutEffect(effectElement,curSubLayer,frameId)
            end
        end
    end
    ,
    effectModelRandomlyPlayOnce = function (this,effectElements,frameId)
        local pick = math.random(#effectElements)
        local effectElement = effectElements[pick]
        local curSubLayer = this:getLayerByName(effectElements[pick].name)
        if(effectElement.type == "FaceStickerElement" or
            effectElement.type == "ForeStickerElement" or
            effectElement.type == "CatStickerElement" or
            effectElement.type == "DogStickerElement" ) then
            effectHandler.setStickerEffect(effectElement,curSubLayer,frameId)
        elseif (effectElement.type == "BeautyMakeupElement") then
            effectHandler.setMakeupEffect(effectElement,curSubLayer,frameId)
        else
            effectHandler.setLutEffect(effectElement,curSubLayer,frameId)
        end
    end
}



triggerhandler = {

    nodeTrigged = function (this,portId,eventMessage)
        -- 1. delete current port
        currentPortPool = removeElementByKey(currentPortPool,portId)
        -- 2.set effects params
        local node = networkHandler.getChildNodebyPortId(portId)

        effectHandler.setEffectModelParams(this,node,eventMessage)
        -- 3.insert child ports
        local childPorts = networkHandler.getChildPortsByPortId(portId)
        print("childports--------   ",#childPorts)

        if(#childPorts > 0)
        then
            for i,v in pairs(childPorts)
            do
                currentPortPool[v] = {}
            end
        end

        -- leaf port needs no hanle

    end
    ,
    updatePortPool = function(portId,eventCode)
        -- add unique evenetcode to table
        local flag = true

        for i, v in pairs(currentPortPool[portId])
        do
            if(v == eventCode)
            then
                flag = false
                break
            end
        end

        if(flag)
        then
            table.insert(currentPortPool[portId],eventCode)
        end
    end
    ,
  handleFaceTrigger = function(this, trigSubType,eventMessage)
        for key, value in pairs(currentPortPool)
        do
            local triggerGroup = networkHandler.getTriggerGroupByPortId(key)       --  a triggerGroup is not empty by default
            local trigCondition = triggerGroup.trigCondition
            local triggerElements = triggerGroup.triggerElements
            local portId = key
            for i, v in pairs(triggerElements)
            do
                if(v.trigType == TrigType.Face and v.trigSubType == trigSubType)
                then
                    if(trigCondition == TrigCondition.Or)
                    then
                        triggerhandler.nodeTrigged(this,portId,eventMessage)
                    else
                        triggerhandler.updatePortPool(portId,trigSubType)
                        if(#currentPortPool[portId] == #triggerElements) then
                            triggerhandler.nodeTrigged(this,portId,eventMessage)
                        end
                    end

                end
            end
        end
    end
    ,
    handleHandTrigger = function(this, trigSubType,eventMessage)
        for key, value in pairs(currentPortPool)
        do
            local triggerGroup = networkHandler.getTriggerGroupByPortId(key)       --  a triggerGroup is not empty by default
            local trigCondition = triggerGroup.trigCondition
            local triggerElements = triggerGroup.triggerElements
            local portId = key
            for index, v in pairs(triggerElements)
            do
                if(v.trigType == TrigType.Hand and v.trigSubType == trigSubType) then
                    if(trigCondition == TrigCondition.Or) then
                        triggerhandler.nodeTrigged(this,portId,eventMessage)
                    else
                        triggerhandler.updatePortPool(portId,trigSubType)
                        if(#currentPortPool[portId] == #triggerElements) then
                            triggerhandler.nodeTrigged(this,portId,eventMessage)
                        end
                    end

                end
            end
        end
    end
}

------------------------------------
---    event entry
------------------------------------
EventHandleHelper = {

}
EventHandles = {
    -- this is one effect item
    handleTouchEvent = function (this, eventCode, eventMessage)
        fps = eventMessage.fps
        if (eventCode == SLRESdk.EventCode.SLREEvent_TOUCHUP_INSIDE) then
            -- print('lua: touch event')
        end
        return true
    end,
    handleEffectEvent = function (this, eventCode, eventMessage)
        --print("事件-------  ",eventCode)
        fps = eventMessage.fps
        if eventCode >= SLRESdk.EventCode.SLREEvent_FACE_DETECT and eventCode <= SLRESdk.EventCode.SLREEvent_EXPRESSION_MAX then
            trigType = TrigType.Face
            trigSubType = eventCode - SLRESdk.EventCode.SLREEvent_FACE_DETECT
            triggerhandler.handleFaceTrigger(this,trigSubType,eventMessage)
        elseif eventCode >= SLRESdk.EventCode.SLREEvent_HAND_APPEAR and eventCode < SLRESdk.EventCode.SLREEvent_HAND_MAX then
            trigType = TrigType.Hand
            trigSubType = eventCode - SLRESdk.EventCode.SLREEvent_HAND_APPEAR
            triggerhandler.handleHandTrigger(this, trigSubType,eventMessage)
        elseif eventCode == SLRESdk.EventCode.SLREEvent_NEW_FRAME then
            effectHandler.newframeHandle(this, eventMessage)
        elseif eventCode == SLRESdk.EventCode.SLREEvent_TAKE_PHOTO_BEGIN then
            queueManager.takePhotoBegin(this, eventCode, eventMessage)
        elseif eventCode == SLRESdk.EventCode.SLREEvent_TAKE_PHOTO_DIMISS
            or eventCode == SLRESdk.EventCode.SLREEvent_TAKE_PHOTO_END then
            queueManager.takePhotoDismiss(this, eventCode, eventMessage)
        elseif eventCode == SLRESdk.EventCode.SLREEvent_VIDEO_RECORD_BEGIN then
            queueManager.videoRecordBegin(this, eventCode, eventMessage)
        elseif eventCode == SLRESdk.EventCode.SLREEvent_VIDEO_RECORD_DIMISS
            or eventCode == SLRESdk.EventCode.SLREEvent_VIDEO_RECORD_END then
            queueManager.videoRecordDismiss(this, eventCode, eventMessage)
        end
        return true
    end,
    handQueryEvent = function (this, eventCode, eventMessage)
        fps = eventMessage.fps
        if eventCode == SLRESdk.EventCode.SLREEvent_QUERY_NEED_ANI_PHOTO_TAKE then
            return 0
        end
        return 0
    end,
}

------------------------------------
---    build netowrk
------------------------------------



    function buildLuaNetwork()
        local luaNetwork = LuaNetwork:new()
        luaNetwork.repetitive = true
        luaNetwork.resetEffect = true
        luaNetwork.nodeData = 
        (function ()
            local nodes = {}
            
            return nodes
        end)()
    
        luaNetwork.connections = 
        (function ()
            local connections = {}
            
            return connections
        end)()
    
        return luaNetwork
    end
    luaNetwork = buildLuaNetwork()
    